from zope.publisher.browser import BrowserPage
from collective.castle import util


class LoginUrl(BrowserPage):

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        return util.login_URL(self.context)


class Logout(BrowserPage):

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        util.logout(self.context, self.request)



from zope.interface import Interface, implements
from zope.formlib import form
from plone.app.portlets.portlets import base
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from collective.castle import util
from Products.CMFCore.utils import getToolByName


class IPortlet(Interface):
    pass


class Assignment(base.Assignment):
    implements(IPortlet)

    title = u'CAS Log in'


class Renderer(base.Renderer):
    render = ViewPageTemplateFile('login_portlet.pt')

    @property
    def submit_url(self):
        if self.is_logged_in():
            return 'castle_logout'
        else:
            return util.login_URL(self.context)

    @property
    def submit_name(self):
        if self.is_logged_in():
            return u'CAS Log out' # XXX TODO: i18n
        else:
            return u'CAS Log in' # XXX TODO: i18n

    def is_logged_in(self):
        mt = getToolByName(self.context, 'portal_membership')
        return not mt.isAnonymousUser()


class AddForm(base.AddForm):
    form_fields = form.Fields(IPortlet)

    def create(self, data):
        return Assignment()



from Products.CMFCore.utils import getToolByName
from urllib import quote
from Products.statusmessages.interfaces import IStatusMessage
#from zExceptions import NotFound


def URL(context):
    return context.restrictedTraverse('@@plone').getCurrentUrl()


def login_URL_base(context):
    acl_users = getToolByName(context, 'acl_users')
    cas_auth_helpers = acl_users.objectValues(['CAS Auth Helper'])
    if cas_auth_helpers:
        return cas_auth_helpers[0].getLoginURL()
    else:
        return None


def login_query_string(context):
    quoted_here_url = mtool = quote(URL(context), '')
    querystring = '?came_from=%s' % quoted_here_url
    portal = getToolByName(context, 'portal_url')()
    if portal[-1:] == '/':
        portal = portal[:-1]
    service_URL =('%s/logged_in%s' % (portal, querystring))
    return '?service=%s' % quote(service_URL, '')


def login_URL(context):
    base = login_URL_base(context)
    if base is None:
        request = context.request
        IStatusMessage(request).addStatusMessage(u"CAS Login is not available. \
            Please configure CAS ", type="warning")
        return None
    return '%s%s' % (base, login_query_string(context))


def logout(context, request):
    mt = getToolByName(context, 'portal_membership')
    mt.logoutUser(REQUEST=request)
    #if request.has_key('portal_skin'):
    #    context.portal_skins.clearSkinCookie()
    #request.RESPONSE.expireCookie('__ac', path='/')
    IStatusMessage(request).addStatusMessage(u'You are now logged out.',
        type='info') # XXX TODO: i18n the actual message
    portal = getToolByName(context, 'portal_url').getPortalObject().absolute_url()
    return request.RESPONSE.redirect(portal)




import os
from setuptools import setup, find_packages

version = '1.1'


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()


setup(name='collective.castle',
      version=version,
      description="Plone UI for CAS login.",
      long_description=read('README.txt'),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='zope plone CAS PAS',
      author='Derek Richardson/Plone Community',
      author_email='product-developers@lists.plone.org',
      url='https://svn.plone.org/svn/collective/collective.castle',
      license='GPL',
      package_dir={'': 'src'},
      packages=find_packages('src'),
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          # -*- Extra requirements: -*-
          'rwproperty',
          'Products.CAS4PAS',
      ],
      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



from StringIO import StringIO

from zope.interface import Interface
from zope.interface import implements

from zope.component import adapts

from zope.schema import TextLine
from zope.schema import Bool

from zope.formlib import form

from Products.CMFCore.utils import getToolByName

from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.interfaces import IPloneSiteRoot

from Products.PlonePAS.Extensions.Install import activatePluginInterfaces

from plone.fieldsets.form import FieldsetsEditForm
from plone.app.controlpanel.interfaces import IPloneControlPanelForm

from Products.statusmessages.interfaces import IStatusMessage

from rwproperty import getproperty
from rwproperty import setproperty


class ICAS4PASPluginSchema(Interface):

    login_url = TextLine(
        title=_(u"CAS Log in URL"),
        description=_(u""),
        required=True)

    logout_url = TextLine(
        title=_(u"CAS Log out URL"),
        description=_(u""),
        required=True)

    validate_url = TextLine(
        title=_(u"CAS Validate URL"),
        description=_(u""),
        required=True)

    session_var = TextLine(
        title=_(u"Session Variable"),
        description=_(u""),
        required=True)

    use_ACTUAL_URL = Bool(
        title=_(u"Use Actual URL"),
        description=_(u""),
        required=True)


class CASSettingsAdapter(object):

    implements(ICAS4PASPluginSchema)
    adapts(IPloneSiteRoot)

    def __init__(self, context):
        url_tool = getToolByName(context, 'portal_url')
        portal = url_tool.getPortalObject()
        acl_users = portal.acl_users
        cas_auth_helpers = acl_users.objectValues(['CAS Auth Helper'])
        if not cas_auth_helpers:
            cas = acl_users.manage_addProduct['CAS4PAS']
            cas.addCASAuthHelper('cas', 'CAS Auth Helper')
            cas.login_url = 'https://your.cas.server:port/cas/login'
            cas.logout_url = 'https://your.cas.server:port/cas/logout'
            cas.validate_url = 'https://your.cas.server:port/cas/validate'
            cas.session_var = '__ac'
            cas.use_ACTUAL_URL = True
            out = StringIO()
            activatePluginInterfaces(portal, 'cas', out)
            msg = 'Created CAS plugin. %s' % out.getvalue()
            IStatusMessage(context.request).addStatusMessage(msg, 'info')
        else:
            cas = cas_auth_helpers[0]
        self.cas = cas

    @getproperty
    def login_url(self):
        return self.cas.login_url

    @setproperty
    def login_url(self, login_url):
        self.cas.login_url = login_url

    @getproperty
    def logout_url(self):
        return self.cas.logout_url

    @setproperty
    def logout_url(self, logout_url):
        self.cas.logout_url = logout_url

    @getproperty
    def validate_url(self):
        return self.cas.validate_url

    @setproperty
    def validate_url(self, validate_url):
        self.cas.validate_url = validate_url

    @getproperty
    def session_var(self):
        return self.cas.session_var

    @setproperty
    def session_var(self, session_var):
        self.cas.session_var = session_var

    @getproperty
    def use_ACTUAL_URL(self):
        return self.cas.use_ACTUAL_URL

    @setproperty
    def use_ACTUAL_URL(self, use_ACTUAL_URL):
        self.cas.use_ACTUAL_URL = use_ACTUAL_URL


class CASControlPanel(FieldsetsEditForm):

    implements(IPloneControlPanelForm)

    form_fields = form.FormFields(ICAS4PASPluginSchema)

    label = _(u"CAS settings")
    description = _(u"CAS settings for this site.")
    form_name = _(u"CAS settings")

    @form.action(_(u"Save"))
    def save(self, action, data):
        if form.applyChanges(self.context,
                self.form_fields,
                data,
                self.adapters):
            self.status = _(u"Changes saved.")
        else:
            self.status = _(u"No changes made.")

    @form.action(_(u"Cancel"))
    def cancel(self, action, data):
        pass
        # TODO: redirect to plone control panel
